# STI Model representing sets of CanCan::Ability rules
class AuthItem < ActiveRecord::Base
  has_and_belongs_to_many :auth_roles
  class << self
    attr_accessor :options
    def test_run(current_user, user_type)
      subclasses.map do |subclass|
        next if subclass.options[:user_type] != user_type
        ability = Ability.new(current_user)
        begin
          subject_class = subclass.options[:subject_name].constantize
          subject = subclass.options[:subject_name].constantize.order('RANDOM()').first
        rescue NameError
          subject = subclass.options[:subject_name]
        end
        operation = subclass.options[:operation]
        subclass.new.apply ability,current_user, user_type

        # check single
        ability.can operation, subject

        # check scope
        begin
          if subject_class
            subject_class.accessible_by ability, operation
          end
        rescue Exception => e
          raise e.class.new("#{e.message} in #{subclass.name}")
        end
        "#{subclass.name}:#{operation}"
      end.keep_if(&:present?)
    end

    def dynamically_create_auth_item_subclasses(authorization_grant)
      if authorization_grant.respond_to? :subject
        subject_name = authorization_grant.subject
      else
        subject_name = authorization_grant.name.demodulize
      end
      if authorization_grant.respond_to? :subject_title
        subject_title = authorization_grant.subject_title
      else
        begin
          subject_title = subject_name.constantize.model_name.human
        rescue Exception => e
          raise e.class.new("#{e.message} in #{authorization_grant.name}")
        end
      end
      grants = authorization_grant.grants
      class_names = []
      position= 0
      grants.each do |operation,operation_definition|
        operation_definition[:scopes].each do |scope_name,scope_definition|
          scope_definition[:user_types].each do |user_type|
            class_names << do_on_subject_with_scope_as_user_type = "AuthItem"+operation.to_s.camelize + subject_name.to_s.gsub('::', '') + scope_name.to_s.camelize + 'As' + user_type.to_s.demodulize
            unless  Object.const_defined?(do_on_subject_with_scope_as_user_type)
              Object.const_set(do_on_subject_with_scope_as_user_type, Class.new(self) {
                self.options =  {
                    subject_name: subject_name,
                    scope_name: scope_name,
                    operation: operation,
                    title:subject_title,
                    description: operation_definition[:name],
                    condition_description: {
                        user_type => scope_definition[:name]
                    },
                    position: position,
                    user_types: [user_type],
                    user_type: user_type,
                    apply_as: :grant,
                    default_for: scope_definition[:default_for],
                    proc: scope_definition[user_type.to_s.underscore.to_sym]
                }
                def apply(ability,current_user, user_type)
                  if current_user.identifiable_type == user_type.to_s
                    options =  self.class.options
                    if options[:proc].nil?
                      raise Exception.new("Proc is nil in #{self.class.name} #{options.inspect}")
                    end
                    if options[:proc].arity == 1
                      options[:proc].call(ability)
                    end
                    if options[:proc].arity == 2

                      options[:proc].call(ability,current_user.identifiable)
                    end
                    if options[:proc].arity == 3
                      options[:proc].call(ability,current_user.identifiable,current_user)
                    end
                  end
                  false
                end
              })
            end
            position += 1
          end
        end
      end
      class_names
    end

    def subclasses
      subs = []
      Dir[File.join(Rails.root, "app", "authorization","grants", "*.rb")].each do |filename|
        # if does not honor naming conventions, ignore
        dynamically_create_auth_item_subclasses( ("Grants::"+File.basename(filename, ".rb").camelize).constantize).each do |sub|

          subs << sub.constantize # rescue NameError
        end
      end
      subs
    end

    def initialize_subclasses
      subclasses unless @subclasses_initialized
      @subclasses_initialized=true
    end


    # write new auth Items to database and delete removed ones
    #
    # It ensures that for every combination of models, user_types and actions of a Subclass
    # an AuthItem exists in the table auth_items.
    # as AuthItem is an STI Model, the operations are filtered per subclass, so every delete only
    # applies to the scope of the subclass.
    # @return Integer number of deleted rows
    def sync
      existing_ids = []
      options[:user_types].each do |for_user_type|
        existing = where(user_type: for_user_type).take # plus STI scope
        existing = create({user_type: for_user_type }) if existing.blank?
        existing_ids << existing.id
        if existing.new_record? && options[:default_for].include?(for_user_type)
          AuthRole.where(user_type: for_user_type).all.each do |role|
            role.auth_items << existing
            role.save
          end
        end
        if options[:default_for].include?(for_user_type)
          existing.is_default_item=true
        end
        existing.save
      end
      delete_ids = where.not(id: existing_ids).ids
      where(id: delete_ids).delete_all
    end



    def sync_all
      delete = []
      results = ActiveRecord::Base.connection.execute("SELECT * FROM #{table_name}")
      results.each do |record|
        begin
          record['type'].constantize
        rescue NameError
          delete << record['type']
        end
      end
      if delete.any?
        ActiveRecord::Base.connection.execute("DELETE FROM #{table_name} WHERE type IN ('#{delete.join("','")}')")
      end
      subclasses.each(&:sync)
      # too lazy to define fk constraint, not critical
      ActiveRecord::Base.connection.execute <<-SQL
        DELETE FROM auth_items_roles WHERE NOT EXISTS (SELECT 1 FROM auth_items WHERE auth_items.id=auth_items_roles.auth_item_id)
      SQL
    end
  end
  # this creates subclasses when auth item class is loaded
  initialize_subclasses

  def subject_name
    self.class.options[:subject_name]
  end

  def title
    self.class.options[:title]
  end

  def description
    self.class.options[:description]
  end

  def condition_description
    self.class.options[:condition_description][user_type.constantize]
  end

  def operation
    self.class.options[:operation]
  end

  def normalized_operation
    if /read|index/.match operation
      return :read
    end
    if /create|new/.match operation
      return :create
    end
    if /update|edit/.match operation
      return :update
    end
    if /destroy|delete/.match operation
      return :destroy
    end
    return :other
  end

  def operation_bootstrap_glyphicon_suffix
    case normalized_operation
      when :read
        'eye-open'
      when :create
        'plus'
      when :update
        'pencil'
      when :destroy
        'remove'
      else
        'asterisk'
    end
  end

  def apply_as
    self.class.options[:apply_as]
  end

  def position
    self.class.options[:position]
  end

  # see dynamically_create_auth_item_subclasses
  def apply(ability,current_user, user_type)
    raise 'This should be "implemented" by dynamically created subclasses in method dynamically_create_auth_item_subclasses'
  end
end
