class ApplicationController < ActionController::Base

  rescue_from CanCan::AccessDenied do |exception|
    respond_to do |format|
      format.json { head :forbidden }
      format.html { redirect_to root_url, :alert => exception.message }
    end
  end

  # Prevent CSRF attacks by raising an exception.
  # For APIs, you may want to use :null_session instead.
  protect_from_forgery with: :exception

  before_action :run_system_setup, if: :setup_running?
  before_action :authenticate_user!, unless: :setup_running?
  before_action :chained_form_elements

  layout :layout_by_resource

  add_breadcrumb "Dashboard", :user_dashboard_path

  add_flash_types :success, :warning, :danger, :info

  def layout_by_resource
    case
      when setup_running?
        return "setup"
      when devise_controller?
        if controller_name == 'registrations' && ['edit','update'].include?(action_name)
          return 'application'
        else
          return false
        end
      else
        return "application"
    end

  end


  private

  def setup_running?
    return false if SystemSetting.setup_finished?
    running = ! (
        SystemSetting.vsb_registered? &&
        SystemSetting.any? &&
        User.any? &&
        Institution.any?
    )
    if ! running
      SystemSetting.setup_finished!
    end
    return running
  end

  def run_system_setup
    # block everything until a secret has been entered
    if ENV["VSB_SETUP_TOKEN"].nil?
      if File.exists?("#{Rails.root}/config/setup_token_verified")
        ENV["VSB_SETUP_TOKEN"] = '1'
      else
        if ! File.exists?("#{Rails.root}/config/setup_token") || params[:vsb_setup_token] == File.read("#{Rails.root}/config/setup_token")
          FileUtils.touch("#{Rails.root}/config/setup_token_verified")
          ENV["VSB_SETUP_TOKEN"] = '1'
        end
      end
      if ENV["VSB_SETUP_TOKEN"].nil?
        render( :text => 'Setup token invalid', :status => 403, :layout => false) and return false
      end
    end
    # perform redirects to setup actions
    case
      when ! SystemSetting.vsb_registered?
        render( :text => 'Registrierung noch nicht durchgeführt', :status => 403, :layout => false) and return false
      when ! SystemSetting.any?
        unless controller_name == 'system_settings' && ['new','create'].include?(action_name)
          # only available action at this point has to be create system setting
          redirect_to :new_system_setting and return false
        end
      when ! Semester.any?
        render( :text => 'CsbApi::Semester keine vorhanden', :status => 404, :layout => false) and return false
      when ! User.any?
        unless devise_controller? && controller_name == 'registrations' && ['new','create'].include?(action_name)
          # only available action at this point has to be user registration
          redirect_to :new_user_registration and return false
        end
      when ! Institution.any?
        unless controller_name == 'institutions' && ['new','create'].include?(action_name) ||
        controller_name== 'addresses' && request.format.json?
          # only available action at this point has to be create organisational institution
          redirect_to :new_institution and return false
        end
    end
    case
      when User.any?
        if devise_controller? && controller_name == 'registrations' && ['new','create'].include?(action_name)
          # this action must not be available after initial registration, all following users are created on their behalf
          redirect_to :root and return false
        end
      when SystemSetting.any?
        if controller_name == 'system_settings' && ['new','create'].include?(action_name)
          # this action is blocked after installation
          redirect_to :root and return false
        end
    end
  end

  def chained_form_elements
    if params['chained_form_element']
      unless respond_to?(params['chained_form_element'])
        cannot_find_method_alert = "<script type='text/javascript'>alert('Cannot call #{self.class}:#{params['chained_form_element']} to render dependent element. Implement the method and provide the view partial _#{params['chained_form_element']}.html.erb')</script>";
        render text: cannot_find_method_alert and return
      end
      do_render = send(params['chained_form_element'])
      unless do_render === false
        render :partial => params['chained_form_element'], :locals => { :disabled => false } and return
      else
        render status: :not_found and return
      end
    end
  end

  def after_sign_in_path_for(resource)
    user_dashboard_path
  end

  def after_sign_out_path_for(resource_or_scope)
    '/users/sign_in'
  end

  def redirect_to_404_if_not_super_admin
    unless current_user.super_admin?
      return render :file => "#{Rails.root}/public/404.html",  :status => 404
    end
    flash.now[:warning] = "Diese Aktion/Seite ist nur für super_admins aufrufbar!"
  end

  def redirect_to_404_if_not_admin
    unless current_user.super_admin?
      return render :file => "#{Rails.root}/public/404.html",  :status => 404
    end
    flash.now[:info] = "Diese Aktion/Seite sollte nur für administrative Benutzer aufrufbar sein."
  end

end
