class ClientDuplicateRequest < ActiveRecord::Base
  has_many :client_duplicate_responses, dependent: :destroy

  ######################################################################################################################
  ### ClientDuplicateRequest can be subject of the notification module
  ######################################################################################################################
  # That means: there can be system notifications for ClientDuplicateRequests
  include Notificationable

  ######################################################################################################################
  ### ClientDuplicateRequests can trigger create and delete of system notifications
  ######################################################################################################################
  # that means: certain callbacks on ClientDuplicateRequests can result in created or deleted notifications
  include Publishable

  serialize :request, JSON

  GENDER_MALE = 'm'
  GENDER_FEMALE = 'f'

  validates :first_name, presence: true
  validates :last_name, null: false
  validates :bday, null: false
  validates :gender, null: false
  validates :request, null: false #  POST request (create client)

  enum result_code: [ :request_initial, :request_failed, :request_retry, :request_performed, :request_finished ]

  after_create do
    SendClientDuplicateRequestJob.perform_later(self)
    FinishClientDuplicateRequestJob.set(wait: 10.seconds).perform_later(self) # overall timeout
    true
  end

  after_update do
    if all_duplicates_received?
      FinishClientDuplicateRequestJob.perform_now(self)
    end
    true
  end

  def has_any_duplicates?
     ClientDuplicateResponse.where(client_duplicate_request_id: id).count != 0
  end

  def all_duplicates_received?
    announced_client_duplicate_responses_count.present? &&
    announced_client_duplicate_responses_count > 0 &&
    announced_client_duplicate_responses_count <= ClientDuplicateResponse.where(client_duplicate_request_id: id).count
  end

  def reset!
    ClientDuplicateResponse.where(client_duplicate_request_id: id).delete_all
    update!({client_duplicate_responses_count:0,result_code:0,announced_client_duplicate_responses_count:nil,retry_count:0})
  end

  def requesting_institution
    Institution.where(id: request['institution_id']).take # refactor: foreign key with constraint, do not fetch from stale post params
  end

  class << self
    def build_from_client_params(client, client_params)
      _gnd = GENDER_MALE if Client.genders[client.gender] == Client::ENUM_GENDER_MALE
      _gnd = GENDER_FEMALE if Client.genders[client.gender] == Client::ENUM_GENDER_FEMALE
      new(request: client_params, bday: client.bday, first_name: client.first_name, last_name: client.last_name, gender: _gnd)
    end

    # gender [:m,:f]
    # bday Date
    # first and last string
    def get_possible_duplicates(first_name,last_name,gender,bday)
      matches = []
      ClientDuplicateRequest.where(bday: bday) .where(gender: gender==:m ? GENDER_MALE : GENDER_FEMALE).find_each(batch_size: 500) do |c|
        if c.last_name.similar?(last_name) && c.first_name.similar?(first_name)
          matches << c
        end
      end
      return matches
    end
  end



end
