class ClientGroup < ActiveRecord::Base

  #### Eine Gruppe von Klienten ("Klasse")

  #--- Concerns
  include Notificationable
  include Publishable

  #--- Associations
  belongs_to :teacher
  belongs_to :institution
  belongs_to :semester

  # alle Schüler die jemals in dieser Klasse waren (ohne die, die in der Klasse sind)
  has_many :career_items, ->{ distinct.client_group_changes }, dependent: :nullify # refs #192
  has_many :former_clients, ->{ where('clients.client_group_id != career_items.client_group_id') }, through: :career_items, source: :client

  # all teacher_institution_relations where the institution is the same as the clients institution ( Needed by auth )
  has_many :teacher_institution_relations, foreign_key: :institution_id, primary_key: :institution_id

  # all institution hierarchies were the parent_id is parent institution of the clients institution ( Needed by auth )
  has_many :institution_hierarchies, foreign_key: :child_id,  primary_key: :institution_id

  # alle schüler, die in der klasse sind
  has_many :clients, dependent: :nullify # refs #192

  # Verlauf der Klassenleitung
  has_many :client_group_histories, dependent: :destroy # refs #192
  has_many :former_teachers, -> (client_group) { where('client_group_histories.end_date IS NOT NULL') }, through: :client_group_histories, source: :teacher

  has_and_belongs_to_many :educational_course_rules, class_name: '::EducationalCourseRule',
                          inverse_of: :client_groups, autosave: true

  ######################################################################################################################
  ### Klassenportfolio
  ######################################################################################################################

  has_many :portfolio_items,
           through: :running_portfolio,
           autosave: false,
           validate: false,
           dependent: :destroy

  has_many :portfolios, as: :subject,
           class_name: '::Portfolio::Portfolio',
           inverse_of:  :subject,
           autosave: true

  belongs_to :running_portfolio,
             foreign_key: :running_portfolio_id,
             class_name: '::Portfolio::Portfolio'

  belongs_to :predecessor_client_group,
             class_name: '::ClientGroup',
             foreign_key: 'predecessor_client_group_id'

  alias_attribute :portfolio_source_client_group_id, :predecessor_client_group_id


  before_create do
    if portfolios.blank?
      build_new_portfolio
    end
    true
  end

  after_create do
    if running_portfolio.blank?
      update_attribute(:running_portfolio_id, portfolios.last.id)
    end
    true
  end

  before_save do
    if portfolio_level_changed?
      running_portfolio.update_attribute(:portfolio_level,portfolio_level) unless running_portfolio.blank?
    end
    true
  end

  def create_new_portfolio
    build_new_portfolio
    self.running_portfolio = portfolios.last
    save
    running_portfolio
  end

  def build_new_portfolio
    portfolios.build({
                         rule_set_id: ::SystemSetting.get_instance.load_latest_client_group_portfolio_rules,
                         rule_set_date: ::Date.today,
                         portfolio_level: portfolio_level
                     })
  end




  #--- Scopes
  scope :semester, -> (semester) { where(semester_id: semester.id)}
  scope :active, -> { where(semester_id: SystemSetting.get_instance.semester_id) }
  scope :next_semester, -> { where(semester_id: Semester.following.first.present? ?  Semester.following.first.id : -1 ) }

  #--- Callback definitions
  before_save :handle_teacher_change, if: ->{teacher_id.present?}


  #--- General Validations
  validates :institution_id, :clients_total, :semester_id, presence: true
  validates :teacher_id, uniqueness: {
      scope: [:semester_id,:teacher_id],
      message: "Die ausgewählte Lehrperson ist im ausgewählten Schuljahr bereits in einer anderen Klasse aktiv."
  }, if: ->{teacher_id.present?}

  validates :normal_education_hours, :special_education_hours, numericality: {only_integer: true}

  #--- Conditional Validations
  validate :teacher_institution


  #--- public methods

  def active
    semester_id ==  SystemSetting.get_instance.semester_id
  end



  # gives the teachers from same institution that are not assigned to any client group in current semester
  # NOTE: The teacher assigned to this client group is always included in the result
  def unassigned_teachers
    same_semester_blocked_teachers = ClientGroup.where(semester_id: semester_id)
    same_semester_blocked_teachers = same_semester_blocked_teachers.where('id != ?', id) unless new_record?
    same_semester_blocked_teachers_array = same_semester_blocked_teachers.pluck(:teacher_id)
    teachers = Teacher.joins(:teacher_institution_relations)
    teachers = teachers.where('teacher_institution_relations.institution_id=?', institution_id)
    teachers = teachers.where.not(id: same_semester_blocked_teachers_array) if same_semester_blocked_teachers_array.any?
    teachers
  end


  # get all teachers who are or have been assigned as a secondary / supportive teacher to this client group
  #
  # @return [Array<Teacher>]
  def secondary_teachers
    secondary_teachers = []
    clients.each do |client|
      next if client.teacher.blank?
      unless secondary_teachers.include?(client.teacher)
        secondary_teachers << client.teacher
      end
    end
    secondary_teachers
  end


  #--- private methods
  private
    


    def teacher_institution
      unless TeacherInstitutionRelation.exists?(institution_id: institution_id, teacher_id: teacher_id) || teacher_id.nil?
        errors.add(:teacher_id, "Die ausgewählte Lehrperson arbeitet nicht an der ausgewählten Schule.") and return false
      end
      true
    end

    def handle_teacher_change
      if attribute_changed?(:teacher_id)
        client_group_histories.build(teacher_id: teacher_id)
      end
    end

end
