class Guardian < ActiveRecord::Base

  ENUM_GENDER_MALE = 0
  ENUM_GENDER_FEMALE = 1

  include AddressConcern

  # Personen mit Klientenbezug: Eltern / Erziehungsberechtigte / Vormünder ("parent" war nicht möglich)

  #--- Associations
  has_many :client_guardian_relations, dependent: :destroy, inverse_of: :guardian
  has_many :clients, through: :client_guardian_relations, inverse_of: :guardians
  #has_many :portfolio_item_involvements, class_name: '::Portfolio::Item', through: :portfolio_item_involved_guardians

  has_many :portfolio_item_diagnostic_informal_result_guardians,
           class_name: '::Portfolio::DiagnosticInformalResultGuardian',
           foreign_key: :guardian_id,
           inverse_of: :guardian

  has_many :portfolio_item_diagnostic_informal_results,
           class_name: '::Portfolio::DiagnosticInformalResult',
           through: :portfolio_item_diagnostic_informal_result_guardians,
           source: :diagnostic_informal_result


  # get all portfolio items where the guardian is involved
  # That means, portfolio items are returned that match one of the following:
  #
  # 1. items where guardian involved in portfolio item
  # 2. items of clients that are children of the guardian ( even if guardian is not involved)
  # 3. items where guardian involved in evaluation of portfolio item
  #
  # OMITTED: items where guardian is author of portfolio item ( this is no use case, guardian cannot create items)
  #
  # @return [ActiveRecordRelation<Portfolio::Item>] collection of portfolio items
  def associated_portfolio_items
    pi_ids = ::Portfolio::Item.find_by_sql(['select distinct pi.id from portfolio_items pi
          inner join portfolio_item_involved_guardians piig on piitg.portfolio_item_id=pi.id
          where piig.guardian_id=?
          UNION
          select distinct p.id from portfolio p
          inner join client_guardian_relations cgr on cgr.client_id=p.subject_id and p.subject_type=?
          where cgr.guardian_id=?
          UNION
          select distinct pi.id from portfolio_items pi
          inner join portfolio_item_targets pit on pit.portfolio_item_id= pi.id
          inner join portfolio_item_evaluations pie on pie.portfolio_item_target_id= pit.id
          inner join evaluation_involved_guardians eig on eig.portfolio_item_evaluation_id=pie.id
          where eig.teacher_id=?',id,'Client', id,id])
    ::Portfolio::Item.where(id: pi_ids)
  end


  #--- Attributes
  enum gender: { "männlich" => ENUM_GENDER_MALE, "weiblich" => ENUM_GENDER_FEMALE }

  #--- General Validations
  validates :first_name, :last_name, presence: true
  validates :email, email: {}
  validates :phone, :mobile, phony_plausible: {normalized_country_code: 'DE'}

  before_save do
    # guardian duplicates are only searched by phonetic hash
    self.first_name_phonetic_code= first_name.phonetic_code
    self.last_name_phonetic_code = last_name.phonetic_code
  end

  validate do
    if marked_for_destruction?
      errors.add(:_destroy,'Die Person hat noch andere Schüler') unless has_single_client?
    end
  end

  def has_single_client?
    client_guardian_relations.to_a.delete_if(&:marked_for_destruction?).size <= 1
  end

  def system_notifications
    portfolio_item_ids = associated_portfolio_items.ids
    portfolio_item_condition = '( notificationable_type=? AND notificationable_id IN (?) )'
    ::Notification.only_showable.where("#{portfolio_item_condition}", 'Portfolio::Item', portfolio_item_ids)
  end

  def full_name
    first_name + " " + last_name
  end

  def full_name_with_clients(number=3)
    first_name + " " + last_name + " (#{clients.take(number).map(&:full_name).join(', ')}) "
  end



end
