module Portfolio
  class ItemAdvancementPlan < ::ActiveRecord::Base
      self.table_name = 'portfolio_item_advancement_plans'
      self.inheritance_column = nil

      include ::Publishable

      belongs_to :portfolio_item,
                 class_name: '::Portfolio::Item',
                 foreign_key: :portfolio_item_id,
                 inverse_of: :advancement_plans
      delegate :portfolio_subject, to: :portfolio_item

      belongs_to :advancement_plan,
                 class_name: '::AdvancementPlan',
                 foreign_key: :advancement_plan_id,
                 inverse_of: :portfolio_item_advancement_plans

      belongs_to :advancement_plan_borrow,
                 class_name: '::AdvancementPlanBorrow',
                 foreign_key: :advancement_plan_borrow_id,
                 inverse_of: :portfolio_item_advancement_plan

      scope :borrow_required, -> { where('advancement_plan_borrow_id IS NULL') }

      # for handling remote  catalog ids. used for html form processing
      # @see #set_advancement_plan_from_catalog_id
      attr_accessor :catalog_id

      validates_presence_of :advancement_plan
      validates_presence_of :portfolio_item

      validates :begin, presence: true
      validates :begin, date: true
      validates :begin, chronological_order: {fields: ['portfolio_item.start', :begin]}
      validates :content, length: { minimum: 50 }, :allow_blank => true # refs #261

      before_validation :set_advancement_plan_from_catalog_id

      # get the advancement_plans that are allowed/availiable for the portfolio item
      #
      # the advancement_plan are always fetched using the csb api. The returned advancement_plans may or may not
      # exist locally.
      #
      # @return [Array<AdvancementPlan>]
      def available_remote_advancement_plans
        ::CsbApi::AdvancementPlan.all.map(&:build_model)
      end

      # @return [Array<AdvancementPlan>]
      def available_own_institution_advancement_plans
        ::AdvancementPlan
            .joins(:advancement_plan_institution_stocks)
            .where('advancement_plan_institution_stocks.institution_id=?', portfolio_subject.institution_id)
            .where('advancement_plan_institution_stocks.stock > 0')
            .all
      end

      # @return [Array<AdvancementPlan>]
      def available_other_institution_advancement_plans
        ::AdvancementPlan
            .joins(:advancement_plan_institution_stocks)
            .where('advancement_plan_institution_stocks.institution_id = ?', portfolio_subject.institution_id)
            .where('advancement_plan_institution_stocks.stock = 0')
            .all
      end

      # get advancement_plans for use in grouped_collection_select
      #
      # all advancement_plans are fetched using the csb api. The returned advancement_plans may or may not
      # exist locally.
      #
      # @return [Array<GroupedOptionTag>]
      def available_remote_advancement_plans_grouped
        [#GroupedOptionTag.new(portfolio_subject.available_remote_advancement_plan_suggestions,'Vorschläge'),
            GroupedOptionTag.new(available_own_institution_advancement_plans, "An #{portfolio_subject.institution.name}"),
            GroupedOptionTag.new(available_other_institution_advancement_plans, "An anderen Einrichtungen")]
      end

      private
      # sets the diagnostic association using accessor catalog_id
      #
      # This is executed as before_validate callback, if validation is skipped, it must be called manually
      #
      # the diagnostic is only set when the catalog_id is set and no diagnostic is associated yet
      #
      # if the diagnostic exists locally, the existing record is set.
      # if not exist locally, the csb api is queried and new diagnostic with all dependencies is built and set
      def set_advancement_plan_from_catalog_id
        if catalog_id.present? && (advancement_plan.blank? && advancement_plan_id.blank?)
          self.advancement_plan = ::AdvancementPlan.where(catalog_id: catalog_id).take
          if self.advancement_plan.blank?
            remote = ::CsbApi::AdvancementPlan.find(catalog_id).first
            self.advancement_plan = remote.build_model unless remote.nil?
          end
        end
      end


      class GroupedOptionTag
        attr_accessor :group, :group_label

        def initialize(group, label)
          @group=group; @group_label=label
        end
      end
    end
end

