module Portfolio
  class ItemDiagnostic < ::ActiveRecord::Base
      self.table_name = 'portfolio_item_diagnostics'
      self.inheritance_column = nil

      include ::Portfolio::Concerns::DiagnosticResultConcern
      include ::Publishable
      self.accepts_formal_results = true

      belongs_to :portfolio_item,
                 class_name: '::Portfolio::Item',
                 foreign_key: :portfolio_item_id,
                 inverse_of: :diagnostics
      delegate :portfolio_subject, to: :portfolio_item
      delegate :closed?, to: :portfolio_item

      belongs_to :diagnostic,
                 class_name: '::Diagnostic',
                 foreign_key: :diagnostic_id,
                 inverse_of: :portfolio_item_diagnostics


      belongs_to :diagnostic_borrow,
                 class_name: '::DiagnosticBorrow',
                 foreign_key: :diagnostic_borrow_id,
                 inverse_of: :portfolio_item_diagnostic


      scope :borrow_required, -> { where('diagnostic_borrow_id IS NULL') }

      alias_attribute :start, :begin

      # for handling remote diagnostic catalog ids. used for html form processing
      # @see #set_diagnostic_from_catalog_id
      attr_accessor :catalog_id

      validates_presence_of :diagnostic

      validates_presence_of :portfolio_item
      validates :begin, presence: true
      validates :begin, date: true
      validates :begin, chronological_order: {fields: ['portfolio_item.start',:begin]}
      validates :content, length: { minimum: 50 }, :allow_blank => true # refs #261

      before_validation :set_diagnostic_from_catalog_id




      # get the diagnostics that are allowed/availiable for the portfolio item
      #
      # the diagnostics are always fetched using the csb api. The returned diagnostics may or may not
      # exist locally.
      #
      # @return [Array<Diagnostic>]
      def available_remote_diagnostics
        ::CsbApi::Diagnostic.all.map(&:build_model)
      end


      # @return [Array<Diagnostic>]
      def available_at_subjects_institution
        ::Diagnostic.distinct
            .where('diagnostics.min_level <= ? AND diagnostics.max_level >= ?',portfolio_subject.portfolio_level,portfolio_subject.portfolio_level)
            .joins(:diagnostic_institution_stocks)
            .where('diagnostic_institution_stocks.institution_id =?',portfolio_subject.institution_id)
            .where('diagnostic_institution_stocks.stock > 0')
      end
      alias :available_at_clients_institution :available_at_subjects_institution

      # @return [Array<Diagnostic>]
      def available_at_other_institutions
        ::Diagnostic.distinct
            .where('diagnostics.min_level <= ? AND diagnostics.max_level >= ?',portfolio_subject.portfolio_level,portfolio_subject.portfolio_level)
            .joins(:diagnostic_institution_stocks)
            .where('diagnostic_institution_stocks.institution_id != ?',portfolio_subject.institution_id)
            .where('diagnostic_institution_stocks.stock > 0')
            .where('diagnostics.id NOT IN (?)', available_at_subjects_institution.ids)
      end


      # get diagnostics for use in grouped_collection_select
      #
      # all diagnostics are fetched using the csb api. The returned diagnostics may or may not
      # exist locally.
      #
      # @return [Array<GroupedOptionTag>]
      def available_remote_diagnostics_grouped
        [#GroupedOptionTag.new(portfolio_subject.available_remote_diagnostic_suggestions,'Vorschläge'),
            GroupedOptionTag.new(available_at_subjects_institution.all,"An #{portfolio_subject.institution.name}"),
            GroupedOptionTag.new(available_at_other_institutions.all,"An anderen Einrichtungen")]
      end

      # sets the diagnostic association using accessor catalog_id
      #
      # This is executed as before_validate callback, if validation is skipped, it must be called manually
      #
      # the diagnostic is only set when the catalog_id is set and no diagnostic is associated yet
      #
      # if the diagnostic exists locally, the existing record is set.
      # if not exist locally, the csb api is queried and new diagnostic with all dependencies is built and set
      def set_diagnostic_from_catalog_id
        if catalog_id.present? && ( diagnostic.blank? && diagnostic_id.blank? )
          self.diagnostic = ::Diagnostic.where(catalog_id: catalog_id).take
          if self.diagnostic.blank?
            remote = ::CsbApi::Diagnostic.includes('diagnostic-images','diagnostic-norms','diagnostic-characteristics').find(catalog_id).first
            self.diagnostic = remote.build_model unless remote.nil?
          end
        end
      end

      private



      class GroupedOptionTag
        attr_accessor :group,:group_label
        def initialize(group,label)
          @group=group;@group_label=label
        end
      end

    end
end

