module Portfolio
  module Subjects
    class ClientGroup < Base

      def name
        "#{@subject.name}"
      end

      def client_group
        @subject
      end

      def client_group_id
        @subject.id
      end

      def is_class_teacher?(teacher)
        teacher.id == @subject.teacher_id
      end

      def is_special_ed?(teacher)
        @subject.clients.each do |c|
          return true if c.teacher_id=teacher.id
        end
        return false
      end

      def relation_description_for_guardian(guardian)
        k_ids = ::Client.where(client_group_id: @subject.id).ids
        rel = guardian.client_guardian_relations.where(client_id: k_ids).all.map do |cgr|
          "#{cgr.category}: #{cgr.client.first_name} #{cgr.client.last_name}"
        end
        rel.join(', ')  unless rel.blank?
      end

      # get the advancement_plans for diagnostics that are allowed/availiable for the portfolio item
      #
      # the advancement_plans are always fetched using the csb api. The returned advancement_plans may or may not
      # exist locally.
      #
      # @return [Array<AdvancementPlan>]
      def available_remote_advancement_plan_suggestions
        ::CsbApi::AdvancementPlanSuggestion.where(client_ids: @subject.clients.ids).all.map(&:build_model)
      end


      # get the suggestions for diagnostics that are allowed/availiable for the portfolio item
      #
      # the diagnostics are always fetched using the csb api. The returned diagnostics may or may not
      # exist locally.
      #
      # The returned records do not include the dependecies of the diagnostics, like characteristics, norms, etc.
      # @see #Diagnostic.build_from_remote for getting full populated Diagnostic
      #
      def available_remote_diagnostic_suggestions
        ::CsbApi::DiagnosticSuggestion.where(client_ids: @subject.clients.ids).all.map(&:build_model)
      end

      # get the records that are  allowed to be set as involved guardians for this portfolio
      #
      # for base items, guardians of clients at the same institution as the portfolio are returned
      # this is used for the select involved guardians field in portfolio item create/update form
      #
      # @return [ActiveRecord::Associations::CollectionProxy<::Guardian>]
      def portfolio_item_available_involved_guardians(portfolio_item)
        _records = ::Guardian.includes(:clients).references(:clients)
                       .where('clients.client_group_id=? OR guardians.id IN (?)',@subject.id, portfolio_item.involved_guardians.ids)
        return _records.distinct(:id)
      end

      # get the records that are allowed to be set as involved supporters
      # which are supporters related all clients in the same institution as the portfolio
      #
      # this is used by the select involved supporters field in portfolio item create/update form
      # @return [ActiveRecord::Associations::CollectionProxy<::Supporter>]
      def portfolio_item_available_involved_supporters(portfolio_item)
        _records = ::Supporter.joins(:clients ).where('clients.client_group_id=? OR supporters.id IN (?)',@subject.id,portfolio_item.involved_supporters.ids)
        _records = _records.where('supporters.id <> ?',portfolio_item.author.id) if portfolio_item.author.class.name == 'Supporter'
        return _records.distinct(:id)
      end

      # get the records that are allowed to be set as involved supporters
      # which are supporters related to the client
      #
      # this is used by the select field for evaluation involved supporters  in portfolio item create/update form
      # @return [ActiveRecord::Associations::CollectionProxy<::Supporter>]
      def evaluation_available_involved_guardians(portfolio_item_evaluation)
        _records = ::Guardian.joins(:clients).where('clients.client_group_id =? OR guardians.id IN (?)',portfolio_item_evaluation.portfolio.subject.id,portfolio_item_evaluation.portfolio_item.involved_guardians.ids)
        return _records.distinct(:id)
      end

      # get the records that are allowed to be set as involved supporters
      # which are supporters related to the client
      #
      # this is used by the select field for evaluation involved supporters  in portfolio item create/update form
      # @return [ActiveRecord::Associations::CollectionProxy<::Supporter>]
      def evaluation_available_involved_supporters(portfolio_item_evaluation)
        _records =  ::Supporter.joins(:clients).where('clients.client_group_id=? OR supporters.id IN (?)',portfolio_item_evaluation.portfolio.subject.id,portfolio_item_evaluation.portfolio_item.involved_supporters.ids )
        return _records.distinct(:id)
      end

      def allows_guardians?(portfolio_item)
        false
      end

      def subject_path
        routes = Rails.application.routes.url_helpers
        routes.client_group_path(@subject)
      end


      def resolve_notifications_path
        routes = Rails.application.routes.url_helpers
        return routes.select_portfolio_item_date_client_groups_path(client_group_id: @subject.id)
      end

      def select_portfolio_item_date_path
        routes = Rails.application.routes.url_helpers
        routes.select_portfolio_item_date_client_groups_path(client_group_id: @subject.id)
      end

      def redirect_after_item_create
        routes = Rails.application.routes.url_helpers
        routes.client_group_path(@subject)
      end

      def portfolio_title
        'Klassenportfolio'
      end

    end
  end
end