class Supporter < ActiveRecord::Base

  ENUM_GENDER_MALE = 0
  ENUM_GENDER_FEMALE = 1

  #--- Associations
  belongs_to :institution
  has_many :client_supporter_relations, dependent: :destroy, inverse_of: :supporter
  has_many :clients, through: :client_supporter_relations, inverse_of: :supporters

  has_many :portfolio_item_diagnostic_informal_result_supporters,
           class_name: '::Portfolio::DiagnosticInformalResultSupporter',
           foreign_key: :supporter_id,
           inverse_of: :supporter

  has_many :portfolio_item_diagnostic_informal_results,
           class_name: '::Portfolio::DiagnosticInformalResult',
           through: :portfolio_item_diagnostic_informal_result_supporters,
           source: :diagnostic_informal_result

  #--- Attributes
  enum gender: { "männlich" => ENUM_GENDER_MALE, "weiblich" => ENUM_GENDER_FEMALE }

  # be really careful with the keys here, there are methods mixed in based on them, i.e. letting an empty key ("") override the ! operator, which is awful to debug
  enum title: { " " => 0, "Dr." => 1, "Prof." => 2, "Prof. Dr." => 3 }



  attr_accessor :institution_choice

  # get all portfolio items where a supporter is involved
  #
  #
  # 1. items where supporter involved in portfolio item
  # 2. items where supporter involved in evaluation of portfolio item
  # 3. items where supporter is author of portfolio item
  #
  # @return [ActiveRecordRelation<Portfolio::Item>]  collection
  def associated_portfolio_items
    pi_ids =::Portfolio::Item.find_by_sql(['select distinct pi.id from portfolio_items pi
            inner join portfolio_item_involved_supporters piis on piis.portfolio_item_id=pi.id
            inner join supporters s on s.id=piis.supporter_id
            where piis.supporter_id=?
            UNION
            select distinct pi.id from portfolio_items pi on
            inner join portfolio_item_targets pit on pit.portfolio_item_id= pi.id
            inner join portfolio_item_evaluations pie on pie.portfolio_item_target_id= pit.id
            inner join evaluation_involved_supporters eis on eis.portfolio_item_evaluation_id=pie.id
            where eis.supporter_id=?
            UNION
            select distinct pi.id from portfolio_items pi
            where pi.author_type=? and pi.author_id=?', id,id,'Supporter',id])
    ::Portfolio::Item.where(id: pi_ids)
  end

  def self.associated_clients
    client_ids = ::Client.find_by_sql(['select distinct c.id from clients c
      inner join portfolio_items pi on pi.subject_id=c.id and pi.subject_type=?
      inner join portfolio_item_involved_supporters piis on piis.portfolio_item_id=pi.id
      inner join supporters s on s.id=piis.supporter_id
      where piis.supporter_id=?
      UNION
      select distinct c.id from clients c
      inner join client_groups cg on c.client_group_id=cg.id
      inner join portfolio_items pi on pi.subject_id=cg.id and pi.subject_type=?
      inner join portfolio_item_involved_supporters piis on piis.portfolio_item_id=pi.id
      inner join supporters s on s.id=piis.supporter_id
      where piis.supporter_id=?
      UNION
      select distinct c.id from clients c
      inner join portfolio_items pi on pi.subject_id=c.id  and pi.subject_type=?
      inner join portfolio_item_targets pit on pit.portfolio_item_id= pi.id
      inner join portfolio_item_evaluations pie on pie.portfolio_item_target_id= pit.id
      inner join evaluation_involved_supporters eis on eis.portfolio_item_evaluation_id=pie.id
      where eis.supporter_id=?
      UNION
      select distinct c.id from clients c
      inner join client_groups cg on c.client_group_id=cg.id
      inner join portfolio_items pi on pi.subject_id=cg.id  and pi.subject_type=?
      inner join portfolio_item_targets pit on pit.portfolio_item_id= pi.id
      inner join portfolio_item_evaluations pie on pie.portfolio_item_target_id= pit.id
      inner join evaluation_involved_supporters eis on eis.portfolio_item_evaluation_id=pie.id
      where eis.supporter_id=?
      UNION
      select distinct c.id from clients c
      inner join client_supporter_relations csr on csr.client_id=c.id
      where csr.supporter_id=?
      UNION
      select distinct c.id from clients c
      inner join portfolio_items pi on pi.subject_id=c.id  and pi.subject_type=?
      where pi.author_type=? and pi.author_id=?
      UNION
      select distinct c.id from clients c
      inner join client_groups cg on c.client_group_id=cg.id
      inner join portfolio_items pi on pi.subject_id=cg.id  and pi.subject_type=?
      where pi.author_type=? and pi.author_id=?', 'Client', id,'ClientGroup', id,'Client',id,'ClientGroup',id,id,'Client','Supporter',id,'ClientGroup','Supporter',id])
    ::Client.where(id: client_ids)
  end

  #--- General Validations
  validates :first_name, :last_name, :gender, :position, :phone, :email, presence: true
  validates :email, email: {}
  validates :phone, :mobile, :fax, phony_plausible: {normalized_country_code: 'DE'}

  #--- Conditional Validations
  validate :institution_is_present_if_checked

  def system_notifications
    portfolio_item_ids = associated_portfolio_items.ids
    portfolio_item_condition = '( notificationable_type=? AND notificationable_id IN (?) )'
    ::Notification.only_showable.where("#{portfolio_item_condition}", 'Portfolio::Item', portfolio_item_ids)
  end

  def full_name
    first_name + " " + last_name
  end


  private
  def institution_is_present_if_checked
    if institution_choice == '2' && institution.nil?
      errors.add(:institution, "Bitte eine Einrichtung auswählen.") and return false
    end
    true
  end
end