class SystemSetting < ActiveRecord::Base

  include Publishable

  belongs_to :semester # the currently active semester

  validates :region, :name, presence: true
  validates :semester_id, presence: true, on: :update
  validate :semester_id, on: :update do
    unless semester_id.blank?
      # can only set the current semester
      unless semester.start <= Date.today() && semester.finish >= Date.today()
        unless Rails.env == 'development' # in development mode, it is possible to set a new semester, that has not begun yet
          errors.add(:semester_id, 'Dieses Schuljahr ist nicht aktuell.')
        end
      end
      # can only set when every client get his class
      errors.add(:semester_id, 'Es gibt noch Schüler ohne Klassen') if ::Client.where(next_semester_client_group_id: nil, is_former_client: false).any?
    end
  end

  class << self

    # @return SystemSetting
    def get_instance
      @instance ||= first
    end
    def setup_finished?
      true==get_instance.try{ setup_finished }
    end

    def setup_finished!
      true==get_instance.try{update_column(:setup_finished,true)}
    end

    def vsb_registered?
      CsbApi::VsbRegistration.where(vsb_id: Rails.configuration.x.local_env["VSB_ID"]).any?
    rescue JsonApiClient::Errors::ConnectionError => e
      Rails.logger.error("Cannot access csb_api. Is local_env.yml CSB_API_ENDPOINT correct? #{e}")
      false
    end
  end

  def bugfix_version
    _split_version(version,:bugfix)
  end

  def minor_version
    _split_version(version,:minor)
  end


  def major_version
    _split_version(version,:major)
  end

  def vsb_version_lesser_than?(compare_version)
    major = _split_version(compare_version,:major)
    minor = _split_version(compare_version,:minor)
    bugfix = _split_version(compare_version,:bugfix)
    (major_version < major) ||
    (major_version == major && minor_version < minor) ||
    (major_version == major && minor_version == minor && bugfix_version < bugfix)
  end


  def vsb_id
    Rails.configuration.x.local_env["VSB_ID"]
  end

  def available_regions
    CsbApi::Region.all.map{|r|r.name}
  end

  def can_perform_semester_change?
    semester_finished = semester.finish <= Date.today()
    if Rails.env == 'development'
      semester_finished = true # one can activate a semester that has not begun only in development environment
    end
    semester_finished && Semester.following.any? && ! ::Client.where(next_semester_client_group_id: nil, is_former_client: false).any?
  end

  def cannot_perform_semester_change_reason
    if ::Client.where(next_semester_client_group_id: nil, is_former_client: false).any?
      return "Es gibt noch Schüler ohne Klasse"
    end
    if !(semester.finish <= Date.today()) && Rails.env != 'development'
      return "Das aktuelle Schuljahr ist noch nicht vorbei"
    end
    if ! (Semester.following.any?)
      return "Es wurde noch kein Folgesschuljahr gefunden"
    end
  end


  # load or complement the latest ruleset from csb
  def load_latest_portfolio_rules
    _result= CsbApi::PortfolioRuleSet.where(region: region,applicable_from: Date.today, subject_type: 'client').order(applicable_from: :desc).first
    _rule_set_id = _result.rule_set_id
    _remotes =  _result.build_definitions
    _locals = {}
    _changed_rulesets ={}
    # which of them need to be updated?
    ActiveRecord::Base.transaction do
      _remotes.each do |r|
        if exists = r.class.where(id: r.id).take#, portfolio_rule_set_id: r.portfolio_rule_set_id).take
          r.attributes.each_pair {|k,v| exists.assign_attributes({k => v}) unless v.nil? }
          _changed_rulesets[exists.portfolio_rule_set_id]  = exists.portfolio_rule_set_id_was if exists.portfolio_rule_set_id_changed?
          exists.save(validate: false)
          _locals[r.class] ||= []
          _locals[r.class] << exists.id
        else
          r.save(validate: false)
          _locals[r.class] ||= []
          _locals[r.class] << r.id
        end
      end
      _changed_rulesets.each do |after,before|
        Portfolio::Portfolio.where(rule_set_id: before).update_all(rule_set_id: after)
      end
    end
    _rule_set_id
  end

  # load or complement the latest ruleset from csb
  def load_latest_client_group_portfolio_rules
    _result= CsbApi::PortfolioRuleSet.where(region: region,applicable_from: Date.today, subject_type: 'client_group').order(applicable_from: :desc).first
    _rule_set_id = _result.rule_set_id
    _remotes =  _result.build_definitions
    _locals = {}
    _changed_rulesets ={}
    # which of them need to be updated?
    ActiveRecord::Base.transaction do
      _remotes.each do |r|
        if exists = r.class.where(id: r.id).take#, portfolio_rule_set_id: r.portfolio_rule_set_id).take
          r.attributes.each_pair {|k,v| exists.assign_attributes({k => v}) unless v.nil? }
          _changed_rulesets[exists.portfolio_rule_set_id]  = exists.portfolio_rule_set_id_was if exists.portfolio_rule_set_id_changed?
          exists.save(validate: false)
          _locals[r.class] ||= []
          _locals[r.class] << exists.id
        else
          r.save(validate: false)
          _locals[r.class] ||= []
          _locals[r.class] << r.id
        end
      end
      _changed_rulesets.each do |after,before|
        Portfolio::Portfolio.where(rule_set_id: before).update_all(rule_set_id: after)
      end
    end
    _rule_set_id
  end

  #  all updated csb api semester records for running and upcoming semesters.
  def load_updated_semesters
    if Semester.any?
      csb_api_semesters = CsbApi::Semester.where(updated_at_greater: Semester.maximum('csb_updated_at'), finish_greater_equal: Date.today(), region: region)
      csb_api_semesters.all.each do |csb_api_semester|
        _semester = Semester.where(name: csb_api_semester.name).take
        _semester = Semester.new if _semester.blank?
        csb_api_semester.build_model(_semester).save(validate: false)
      end
    else
      csb_api_semesters = CsbApi::Semester.where(finish_greater_equal: Date.today(), region: region)
      csb_api_semesters.all.each do |csb_api_semester|
        csb_api_semester.build_model.save(validate: false)
      end
    end
  end


  def load_updated_educational_courses
    csb_api_educational_courses = CsbApi::EducationalCourse.includes('educational-course-rules').where(region: region)
    csb_api_educational_courses.all.each do |csb_api_educational_course|
      _course = EducationalCourse.where(name: csb_api_educational_course.name, valid_from: csb_api_educational_course.send('valid-from')).take
      _course = EducationalCourse.new if _course.blank?
      csb_api_educational_course.populate_model(_course).save(validate: false)
    end
  end


  # here all client groups are updated from the next_semester_client_group_ids
  # this is quite expensive, should be be done in background.
  after_update do
    if semester_id_changed? # the next semester has begun
      ::Client.all.each do |client|
        client.client_group_id=client.next_semester_client_group_id
        client.next_semester_client_group_id=nil
        client.save(validate:false)
      end
      ::ClientGroup.semester(semester).all.each do |client_group|
        ::ClientGroup.transaction do
          if client_group.predecessor_client_group.present? && source_client_group = client_group.predecessor_client_group
            if source_client_group.running_portfolio
              # laufendes portfolio übernehmen
              client_group.running_portfolio = source_client_group.running_portfolio
              source_client_group.portfolios.delete(source_client_group.running_portfolio)
              client_group.portfolios << client_group.running_portfolio

              # der alten klasse ein leeres Portfolio geben
              source_client_group.build_new_portfolio unless source_client_group.portfolios.any?
              source_client_group.running_portfolio_id = source_client_group.portfolios.last
              source_client_group.save(validate:false)

              # neue klasse kann alle leeren portfolios verwerfen
              client_group.portfolios.each do |portfolio|
                portfolio.destroy if portfolio.portfolio_items.blank?
              end
              client_group.save(validate:false)
            end
            unless client_group.portfolios.any?
              # letzte prüfung: wenn hier immer noch kein portfolio gesetzt ist, ein neues erzeugen
              client_group.create_new_portfolio
            end
          end
        end
      end
    end
    true
  end

  before_create do
    load_latest_portfolio_rules
    load_updated_semesters
    self.version = File.read(Rails.root.join('version'))
    self.semester = Semester.where('start <= ? AND finish >= ?',Date.today,Date.today).first
    self.semester.present? && ::Portfolio::Rule.any?
  end

  after_create do
    # initially load the categories from CSB
    #CsbApi::TeacherCategory.where(region: region).all.map(&:build_model).map(&:save) unless TeacherCategory.any?
    #CsbApi::TeacherInstitutionCategory.where(region: region).all.map(&:build_model).map(&:save) unless TeacherInstitutionCategory.any?
    CsbApi::InstitutionCategory.where(region: region).all.map(&:build_model).map(&:save) unless InstitutionCategory.any?
    CsbApi::EducationalCourse.includes('educational-course-rules').where(region: region).all.map(&:build_model).map(&:save) unless EducationalCourse.any?
    true
  end

  private
  def _split_version(v,pos)
    pos = 0 if pos == :major
    pos = 1 if pos == :minor
    pos = 2 if pos == :bugfix
    return 0 if v.blank?
    parts =  v.split('.')
    return 0 if parts[pos].blank?
    return parts[pos].to_i
  end

end
