module Portfolio
  # Like a Compositum for portfolio item validators
  # accepts requirements in the following JSON form:
  #
  # Example:
  # [
  #   [
  #    {
  #    "type": "involvement_validator",
  #    "args" :
  #     {
  #      "type":"teacher"
  #     }
  #    },
  #    {
  #     "type": "involvement_validator",
  #       "args" : {
  #        "type":"supporter"
  #       }
  #     }
  #   ],
  #   [
  #    {
  #      "type": "involvement_validator",
  #      "args" : {
  #          "type":"guardian"
  #       }
  #     }
  #   ]
  # ]
  #
  # Is used like:
  #
  # <V> = { type: <name_of_validator_directory>, args: <passed to initalize> }
  # [ [ <V> OR <V> ] AND [ <V> OR <V> ]  ]
  #
  # In the validator directories are classes, which are named like portfolio item types (i.e Base, Client, ClientGroup)
  # The actual validator used is decided by the type of the portfolio item:
  # - use ::Portfolio::ItemValidator if validator not found (which always validates successfully)
  # - user ::Portfolio::<Validatorname>::Base when no specific validator found
  # - user ::Portfolio::<Validatorname>::<Specific> when a validator class named like the item type exists
  #
  #
  class StepValidator < ::ActiveModel::Validator

    def initialize(requirements, requirements_description)
      @requirements = []
      @validates = {}
      @requirements_description = []
      @requirements_description = ::JSON.parse(requirements_description) unless requirements_description.nil?
      unless requirements.nil?
        @requirements = ::JSON.parse(requirements).map do |or_requirements|
          or_requirements.map do |object|
            klass = "::Portfolio::#{validator_name(object['type']).camelize}"
            begin
              instance = klass.constantize.new
              object['args'].each do |k, v|
                instance.send(k+'=', v) if instance.respond_to? k+'='
              end unless object['args'].nil?
              @validates[validator_name(object['type'])] = true
            rescue ::NameError
            end
            instance
          end
        end
      end
    end

    def validates?(name)
      @validates.key?(name) || @validates.key?(validator_name(name))
    end

    # @param item ::Portfolio::Item
    def validate(item)
      index = 0
      @requirements.inject(true) do |all_matched, or_requirements|
        or_requirements_valid = or_requirements.inject(false) do |one_of_them_matched, item_validator|
          one_of_them_matched | item_validator.match?(item)
        end
        unless or_requirements_valid
          item.errors[:base] << @requirements_description[index]
        end
        index += 1
        all_matched & or_requirements_valid
      end
    end

    def hints
      @requirements_description
    end

    private
    def validator_name(name)
      map={
          'advancement_plan' => 'advancement_plan_present_validator',
          'advancement_plan_present' => 'advancement_plan_present_validator',
          'involved_person' => 'involvement_validator',
          'involvement' => 'involvement_validator',
          'diagnostic' => 'diagnostic_present_validator',
          'diagnostic_present' => 'diagnostic_present_validator',
          'measurement' => 'measurement_present_validator',
          'measurement_present' => 'measurement_present_validator',
          'diagnostic_measurement' => 'diagnostic_measurement_present_validator',
          'diagnostic_measurement_present' => 'diagnostic_measurement_present_validator',
          'documents' => 'documents_included_validator',
          'documents_included' => 'documents_included_validator',
          'evaluation' => 'evaluation_present_validator',
          'evaluation_present' => 'evaluation_present_validator',
          'evaluation_time_span' => 'evaluation_time_span_validator',
          'parent_teacher_conference' => 'request_parent_teacher_conference_validator',
          'request_parent_teacher_conference' => 'request_parent_teacher_conference_validator',
          'management_conference' => 'request_management_conference_validator',
          'request_management_conference' => 'request_management_conference_validator',
          'target_agreement' => 'target_agreements_present_validator',
          'target_agreements' => 'target_agreements_present_validator',
          'target_agreements_present' => 'target_agreements_present_validator',
          'class_diagnostic' => 'class_diagnostic_present_validator',
          'class_diagnostic_present' => 'class_diagnostic_present_validator',
      }
      if map.key?(name)
        return map[name]
      end
      return name
    end

  end

end
